import { v4 as uuidv4 } from 'uuid';
import { TextUtil } from "./utils/text.util";
var textUtil = new TextUtil;
/**
 *
 *
 * @export
 * @class TextParser
 */
var TextParser = /** @class */ (function () {
    function TextParser(htmlText, targetText, options) {
        this.htmlText = '';
        this.targetText = '';
        this.htmlText = textUtil.stringifyHtml(htmlText);
        this.originalTargetedText = textUtil.stringifyHtml(targetText);
        this.targetText = this.originalTargetedText;
        this.targetText = textUtil.beginsWithOpenTag(this.targetText) ? textUtil.removeFirstOpeningTag(this.targetText) : this.targetText;
        this.targetText = textUtil.endsWithClosingTag(this.targetText) ? textUtil.removeLastClosingTag(this.targetText) : this.targetText;
        this.options = options;
        this.highlightedHtmlResult = this.htmlText;
        this.processedTargetedText = this.targetText;
        this.changeRevisions = [];
        this.leftHtmlText = '';
        this.rightHtmlText = '';
        this.insertedWords = '';
    }
    TextParser.prototype.updateChangeRevision = function (changeText, index, id) {
        var _a = this.options.prefix, prefix = _a === void 0 ? '' : _a;
        var revision = {
            id: "" + prefix + id,
            text: changeText,
            startIndex: index,
            currentProcessedSearchText: this.processedTargetedText
        };
        this.changeRevisions.push(revision);
    };
    /**
     *
     *
     * @return {string} The whole html text with the the highleted text
     * wrapped in a span with an id contaoining the prefix passed if the targerted text is present
     * @memberof TextParser
     */
    TextParser.prototype.getHighlightedHtmlResult = function () {
        var targetedIndex = this.htmlText.indexOf(this.targetText);
        if (targetedIndex < 0) {
            return this.htmlText;
        }
        this.leftHtmlText = this.htmlText.substr(0, targetedIndex);
        this.rightHtmlText = this.htmlText.substr(this.targetText.length + targetedIndex);
        return "" + this.leftHtmlText + this.processedTargetedText + this.rightHtmlText;
    };
    /**
     * get c
     *
     * @private
     * @param {number} index
     * @return {number} - index of the targeted text after  open or close span tag
     * @memberof TextParser
     */
    TextParser.prototype.getNewIndexAfterRevisions = function (index) {
        if (index === 0)
            return index;
        index = this.insertedWords.length + index;
        return index;
    };
    TextParser.prototype.addNewSpan = function (isLast) {
        var id = uuidv4();
        var _a = this.options.prefix, prefix = _a === void 0 ? '' : _a;
        var spanText = isLast ? '</span>' : "<span id='" + prefix + id + "'>";
        var span = {
            text: spanText,
            id: id
        };
        return span;
    };
    TextParser.prototype.saveUpdate = function (options) {
        var spanText = options.spanText, index = options.index, id = options.id;
        var leftPartIncludingCharAtIndex = this.processedTargetedText.substr(0, index);
        var rightPart = this.processedTargetedText.substr(index);
        if (leftPartIncludingCharAtIndex === '') {
            this.processedTargetedText = "" + spanText + rightPart;
        }
        else {
            this.processedTargetedText = "" + leftPartIncludingCharAtIndex + spanText + rightPart;
        }
        this.insertedWords += spanText;
        this.updateChangeRevision(spanText, index, id);
    };
    TextParser.prototype.insertSpan = function (index, isLast) {
        if (isLast === void 0) { isLast = false; }
        var _a = this.addNewSpan(isLast), spanText = _a.text, id = _a.id;
        index = this.getNewIndexAfterRevisions(index);
        return this.saveUpdate({ spanText: spanText, index: index, id: id, isBeginning: false });
    };
    TextParser.prototype.parser = function () {
        var seenNonTag = false;
        var seenStartTag = false;
        var charStartIndex = 0;
        for (var i = 0; i < this.targetText.length; i++) {
            var char = this.targetText[i];
            var isCharAtTheLastIndex = (i === this.targetText.length - 1);
            if (!textUtil.beginsWithOpenTag(char)) {
                if (seenStartTag) {
                    if (char === textUtil.checkTagInText().closingSign) {
                        seenNonTag = false;
                        charStartIndex = i;
                        seenStartTag = false;
                        continue;
                    }
                    else {
                        // looking for the next closing tag
                        charStartIndex = i;
                        continue;
                    }
                }
                else {
                    if (!seenNonTag) {
                        charStartIndex = i;
                        seenNonTag = true;
                    }
                    else {
                        if (isCharAtTheLastIndex) {
                            this.insertSpan(charStartIndex);
                            var isLast = true;
                            this.insertSpan((i + 1), isLast);
                            break;
                        }
                        else {
                            // Look ahead for more non tags
                            continue;
                        }
                    }
                }
            }
            else {
                if (seenStartTag) {
                    //<>
                    if (char === textUtil.checkTagInText().closingSign) {
                        seenNonTag = false;
                        charStartIndex = i;
                        seenStartTag = false;
                        continue;
                    }
                    else {
                        // look for closing tag >
                        continue;
                    }
                }
                else {
                    seenStartTag = true;
                    if (seenNonTag) {
                        // insert opening span
                        this.insertSpan(charStartIndex);
                        var isLast = true;
                        //insert closing span
                        this.insertSpan(i, isLast);
                    }
                }
                charStartIndex = i;
                seenNonTag = false;
                /**
                * Keep skipping until we see end tag >
                */
            }
            if (isCharAtTheLastIndex) {
                var isLast = true;
                // Opening span
                this.insertSpan(charStartIndex);
                // Closing
                this.insertSpan(i + 1, isLast);
                break;
            }
        }
        return {
            revisionHistory: this.changeRevisions,
            processTargetedText: this.processedTargetedText,
            totalProcessedText: this.insertedWords,
            highlightedHtmlResult: this.getHighlightedHtmlResult(),
            initialTargetText: this.originalTargetedText,
            formattedTargetedText: this.targetText
        };
    };
    return TextParser;
}());
export { TextParser };
