/**
 * contains methods that helps in formatting the highlighted html text
 * @class TextUtil
 */
var TextUtil = /** @class */ (function () {
    function TextUtil() {
    }
    /**
     * removes all newline in html text
     * @param {string} htmlText - highlighted html text
     * @return {string} the highligthed html text a single line
     * @memberof TextUtil
     */
    TextUtil.prototype.stringifyHtml = function (htmlText) {
        return htmlText.split('\n').join('');
    };
    /**
     *
     * return the sign types
     * @return {object}  - an object containing a greater than '>'sign or a less than sign '<'
     * @memberof TextUtil
     */
    TextUtil.prototype.checkTagInText = function () {
        return {
            openingSign: '<',
            closingSign: '>'
        };
    };
    /**
     *
     * checks the first character in the targeted  html text
     * @param {string} targetedHtmlText
     * @return {boolean} true or false depending on first character in the html
     * text to be higlighted starts with an opening sign '<'
     * @memberof TextUtil
     */
    TextUtil.prototype.beginsWithOpenTag = function (targetedHtmlText) {
        return targetedHtmlText[0] === this.checkTagInText().openingSign;
    };
    /**
    *
    * checks the first character in the targeted  html text
    * @param {string} targetedHtmlText
    * @return {boolean} true or false depending on last character in the html
    * text to be higlighted starts with an closing sign '<'
    * @memberof TextUtil
    */
    TextUtil.prototype.endsWithClosingTag = function (targetedHtmlText) {
        return targetedHtmlText[targetedHtmlText.length - 1] === this.checkTagInText().closingSign;
    };
    //<p1> hello <br> there <p1>
    /**
     *
     *
     * @private
     * @param {string} targetedHtmlText - the html content to be highlighted
     * @param {number} tagStartIndex - the index of the opening sign(if the html targeted text starts with a tag or ends with a tag)
     * @param {number} tagEndIndex - the index of the closing sign
     * @return {string} target html text without the initial open and close tag(if present)
     * @memberof TextUtil
     */
    TextUtil.prototype.abstractTextFromTag = function (targetedHtmlText, tagStartIndex, tagEndIndex) {
        var stringBeforeOpeningTag = targetedHtmlText.substr(0, tagStartIndex);
        var stringAfterClosingTag = targetedHtmlText.substr(tagEndIndex + 1);
        var searchText = "" + stringBeforeOpeningTag + stringAfterClosingTag;
        return searchText;
    };
    ;
    /**
     *
     *
     * @param {string} targetedText - highlighed html text
     * @return {string}  target html text without the first open sign in the first and last tag
     * @memberof TextUtil
     */
    TextUtil.prototype.removeFirstOpeningTag = function (targetedText) {
        var newtargetedText = this.identifyNextOpenTagIndex(targetedText).targetedText;
        if (targetedText[0] === this.checkTagInText().openingSign) {
            return this.removeFirstOpeningTag(newtargetedText);
        }
        else {
            return newtargetedText;
        }
    };
    /**
     * look up for the index of the opening sign '<' in the first open  and last open tag of the highlited html text
     *
     * @private
     * @param {string} targetedText - highlighed html text
     * @return {object}  - index of the opening sign '<' in the first open tag , last open tag and the target html text
     * @memberof TextUtil
     */
    TextUtil.prototype.identifyNextOpenTagIndex = function (targetedText) {
        var tagStartIndex = 0;
        var tagEndIndex = 0;
        var seenStartTag = false;
        for (var i = 0; i < targetedText.length; ++i) {
            if (!seenStartTag && targetedText[i] === this.checkTagInText().openingSign) {
                tagStartIndex = i;
                seenStartTag = true;
                break;
            }
            if (targetedText[i] === this.checkTagInText().closingSign) {
                tagEndIndex = i;
                break;
            }
        }
        targetedText = this.abstractTextFromTag(targetedText, tagStartIndex, tagEndIndex);
        return { tagStartIndex: tagStartIndex, tagEndIndex: tagEndIndex, targetedText: targetedText };
    };
    ;
    /**
     *
     *
     * @param {string} targetedText - highlighed html text
     * @return {string}  target html text without the first close sign  '>' in the first and last tag
     * @memberof TextUtil
     */
    TextUtil.prototype.removeLastClosingTag = function (targetedText, startIndex) {
        if (startIndex === void 0) { startIndex = null; }
        var _a = this.identifyNextClosingTagIndex(targetedText, startIndex), newtargetedText = _a.targetedText, tagStartIndex = _a.tagStartIndex;
        if (targetedText[tagStartIndex - 1] === this.checkTagInText().closingSign) {
            return this.removeLastClosingTag(newtargetedText, tagStartIndex - 1);
        }
        else {
            return newtargetedText;
        }
    };
    /**
     * look up for the index of the  closing sign '>' in the first open  and last open tag of the highlited html text
     *
     * @private
     * @param {string} targetedText - highlighed html text
     * @return {object}  - index of the opening sign '>' in the first open tag , last open tag and the target html text
     * @memberof TextUtil
     */
    TextUtil.prototype.identifyNextClosingTagIndex = function (targetedText, startIndex) {
        var tagStartIndex = 0;
        var tagEndIndex = 0;
        var seenEndTag = false;
        for (var i = startIndex || targetedText.length - 1; i >= 0; --i) {
            if (!seenEndTag && targetedText[i] === this.checkTagInText().closingSign) {
                tagEndIndex = i;
                seenEndTag = true;
            }
            if (targetedText[i] === this.checkTagInText().openingSign) {
                tagStartIndex = i;
                break;
            }
        }
        targetedText = this.abstractTextFromTag(targetedText, tagStartIndex, tagEndIndex);
        return { tagStartIndex: tagStartIndex, tagEndIndex: tagEndIndex, targetedText: targetedText };
    };
    return TextUtil;
}());
export { TextUtil };
